# Zope imports
from zope.interface import Interface
from zope.interface import implements
from zope.component import queryUtility

# CMF imports
from Products.CMFCore.utils import getToolByName

# Five imports
from Products.Five.browser import BrowserView

# rating imports
from iqpp.rating.interfaces import IRatingManager
from iqpp.rating.interfaces import IRatingDefinition
from iqpp.rating.config import MESSAGES

class IRatingView(Interface):
    """Provides methods to rate an object.
    """
    def getRating(self, id, user=None):
        """Returns the rating for rating definition with given id and the 
           authenticated member.
        """
        
    def hasRated(id):
        """Returns True if the authenticated member has already rated for
        given id.
        """

    def rate():
        """Rates object.
        """

    def redirect():
        """Redirects after rating. could be overwritten by 3rd party products 
        to combine rating with other services.
        """
        
class IRatingDetailsView(Interface):
    """Provides methods to retrieve rating details. 
    """    
    def getAmountOfRatings(id):
        """Returns amount of ratings for rating definition with given id.
        """        
    
    def getAverageRating(id):
        """Returns average rating for rating definition with given id.
        """        

    def getRatedScores(id):
        """Returns a dict of all rated scores, whereas the keys are the scores
        and the values are the amount of ratings of the score.
        """
                                
    def getScores(id):
        """Returns all scores of rating definition with given id.
        """        

class IRatingManageView(Interface):
    """Provides methods to manage ratings.
    """
    def getRatings(id):
        """Returns all ratings for rating definition with given id.
        """
    
class RatingView(BrowserView):
    """
    """
    implements(IRatingView, IRatingDetailsView, IRatingManageView)

    def deleteRating(self, id, member_id=None, add_message=True, redirect=True):
        """
        """
        if member_id is None:
            mtool = getToolByName(self.context, "portal_membership")
            member_id = mtool.getAuthenticatedMember().getId()
            
        rm = IRatingManager(self.context)
        result = rm.deleteRating(id, member_id)

        utils = getToolByName(self.context, "plone_utils")

        if add_message == True:
            if result == True:            
                utils.addPortalMessage(MESSAGES["rating-deleted"])
            else:
                utils.addPortalMessage(MESSAGES["rating-not-deleted"])
            
        # redirect
        if redirect == True:
            self.redirect()
        
    def getScores(self, id):
        """
        """        
        rd = queryUtility(IRatingDefinition, id)
        return [s[0] for s in rd.scores]
                    
    def rate(self, add_message=True, redirect=True):
        """
        """                
        rating_id    = self.request.get("rating_id")
        ipaddress    = self.request.get("REMOTE_ADDR")        
        rating_value = self.request.get("rating_value")

        if rating_value == "please_rate":
            url = self.context.absolute_url()
            self.request.response.redirect(url)
        else:
            # get user
            mtool = getToolByName(self.context, "portal_membership")
            user = mtool.getAuthenticatedMember().getId()

            # store rating
            rm = IRatingManager(self.context)
            result = rm.rate(user, rating_id, rating_value, ipaddress)
            self.context.reindexObject()
                                    
            if add_message == True:
                utils = getToolByName(self.context, "plone_utils")
                utils.addPortalMessage(MESSAGES[result])
        
            # redirect
            if redirect == True:
                self.redirect()
        
    def getRating(self, id, user=None):
        """
        """
        if user is None:
            mtool = getToolByName(self.context, "portal_membership")
            user = mtool.getAuthenticatedMember().getId()

        rm = IRatingManager(self.context)
        return rm.getRating(id, user)
        
    def getRatings(self, id):
        """
        """
        ttool = getToolByName(self.context, 'translation_service')
        
        result = []
        rm = IRatingManager(self.context)
        for rating in rm.getRatings(id):
            result.append({
                "user": rating.user,
                "fullname" : self._getFullnameByMemberId(rating.user),
                "value" : rating.value,                
                "timestamp" : ttool.ulocalized_time(rating.timestamp, True),
            })
            
        return result
                        
    def getAverageRating(self, id):
        """
        """
        rm = IRatingManager(self.context)
        return rm.getAverageRating(id)
        
    def getAmountOfRatings(self, id):
        """
        """
        rm = IRatingManager(self.context)
        return rm.getAmountOfRatings(id)

    def getRatedScores(self, id):
        """
        """
        rm = IRatingManager(self.context)
        return rm.getScores(id)
        
    def hasRated(self, id):
        """
        """
        mtool = getToolByName(self.context, "portal_membership")
        user = mtool.getAuthenticatedMember().getId()

        rm = IRatingManager(self.context)
        return rm.hasRated(id, user)        

    def redirect(self):
        """
        """
        url = "%s/view" % self.context.absolute_url()
        self.request.response.redirect(url)

    def _getFullnameByMemberId(self, id):
        """
        """     
        mtool = getToolByName(self.context, "portal_membership")
        creator = self.context.Creator()
        mi = mtool.getMemberInfo(id);
        if mi and mi['fullname']:
            return mi['fullname']

        return id        