# zope imports
from zope.interface import Interface, Attribute
from zope.app.annotation.interfaces import IAttributeAnnotatable
from zope.viewlet.interfaces import IViewletManager
from zope import schema

class IRateable(IAttributeAnnotatable):
    """A marker interface marking an object as being ratable.
    """

class IRatingManager(Interface):
    """An interface which provides methods for rating arbitrary objects.
    """        
    def getAmountOfRatings(id):
        """Returns the total amount of ratings for the definition with given 
        id.
        """
    
    def getAverageRating(id):
        """Returns the average rating value for the definition with given id.
        """

    def getRatings(id):
        """Returns all ratings for the definition with the given id. Returns 
        a list of strings (member ratings) and a float (anonymous).
        """

    def getRating(id, user):
        """Returns a rating for the definition and user. Returns the ratings 
        for anoymous users if given user is None. Returns None if there is no
        rating (e.g. in order to differ zero from no rating).
        """

    def getRatingId():
        """Returns the rating id for an object. If the object has only one 
        rating, in this the passing of an rating id for every single request
        could be omitted.
        """
                
    def getScores(id):
        """Returns a dict of score : amount, whereas the keys are the scores 
        of the definition and the values are the amount of how often the score 
        was rated. If the score wasn't rated at all, it isn't within the dict.
        """

    def hasRated(id, user):    
        """Returns True if the given user has already rated for the given id.
        """

    def rate(id, user, value, ipaddress):
        """Adds a new rating with given informations.
        """

    def rateWithConfirmation(id, value, user):
        """Checks confirmation and calls the rate method.
        """

    def deleteRating(id, user):
        """Deletes the rating for the definition id and user. Returns True if 
        the rating was deleted, otherwise returns False.
        """
        
    def setRating(id, value, user):
        """Sets a value for a specific id and user. 
        """    
        
class IRating(Interface):
    """A single rating for a definition and user.
    """
    def id():
        """
        """

    def value():
        """
        """
    
    def user():
        """
        """
        
    def timestamp():
        """
        """

class IAnonymousRating(IRating):
    """Total rating of anonymous users for a specific definition.
    """
    def hasRated(ipaddress):
        """Returns True if the given ip address has already rated.
        """
            
    def amountOfRatings():
        """Returns the amount of anonymous ratings.
        """
        
    def addValue(self, value, ipaddress):
        """Adds a value and calculates the new value (which is the average 
        rating over all added ratings. Takes also care of the amount of
        ratings.)
        """

class IRatingDefinition(Interface):
    """A definition for a rating.
    """
    def isValidScore(score):
        """Returns true if the given score is valid for this definition.
        """
        
    def getNumericalValue(score):
        """Returns the numerical value for a given score.
        """

class IRated(Interface):
    """A marker interface for the event: user has rated.
    """