# Zope imports
from AccessControl import Unauthorized
from DateTime import DateTime

# zope imports
from zope.interface import directlyProvides
from zope.component import provideUtility, queryUtility

# test imports
from base import RatingTestCase

# iqpp.rating imports
from iqpp.rating.rating import Rating    
from iqpp.rating.rating import AnonymousRating    

class TestRating(RatingTestCase):

    def afterSetUp(self):
        pass

    def testRating(self):
        rating = Rating(id="score", value="1", user="john")
        
        self.assertEqual(rating.id,    "score")
        self.assertEqual(rating.user,  "john")
        self.assertEqual(rating.value, "1")        

class TestAnonymousRating(RatingTestCase):

    def afterSetUp(self):
        self.rating = AnonymousRating(id="score")

    def testInitialization(self):
        self.assertEqual(self.rating.id, "score")
        self.assertEqual(self.rating.user, "anonymous")
        self.assertEqual(self.rating.value, 0)        
        self.assertEqual(self.rating.amountOfRatings, 0)
        self.assertEqual(self.rating.ipaddresses, [])
        
    def testAddValue(self):
        result = self.rating.addValue(2, "2", "127.0.0.1")
        self.assertEqual(result, True)
        self.assertEqual(self.rating.id, "score")
        self.assertEqual(self.rating.user, "anonymous")
        self.assertEqual(self.rating.value, 2.0)        
        self.assertEqual(list(self.rating.scores.keys()), ["2"])                
        self.assertEqual(self.rating.amountOfRatings, 1)  
        self.assertEqual(self.rating.ipaddresses, ["127.0.0.1"])

        result = self.rating.addValue(4, "4", "127.0.0.2")
        self.assertEqual(result, True)
        self.assertEqual(self.rating.id, "score")
        self.assertEqual(self.rating.user, "anonymous")
        self.assertEqual(self.rating.value, 3.0)
        self.assertEqual(list(self.rating.scores.keys()), ["2", "4"])
        self.assertEqual(self.rating.amountOfRatings, 2)  
        self.assertEqual(self.rating.ipaddresses, ["127.0.0.1", "127.0.0.2"])

        result = self.rating.addValue(4, "4", "127.0.0.2")
        self.assertEqual(result, False)

    def testAddValueSameIPAfterOneDay(self):
        result = self.rating.addValue(2, "2", "127.0.0.1")
        self.assertEqual(result, True)
        self.assertEqual(self.rating.id, "score")
        self.assertEqual(self.rating.user, "anonymous")
        self.assertEqual(self.rating.value, 2.0)
        self.assertEqual(list(self.rating.scores.keys()), ["2"])
        self.assertEqual(self.rating.amountOfRatings, 1)  
        self.assertEqual(self.rating.ipaddresses, ["127.0.0.1"])
        
        self.rating._ipaddresses["127.0.0.1"] = DateTime() - 1
        
        result = self.rating.addValue(4, "4", "127.0.0.1")
        self.assertEqual(result, True)
        self.assertEqual(self.rating.id, "score")
        self.assertEqual(self.rating.user, "anonymous")
        self.assertEqual(self.rating.value, 3.0)
        self.assertEqual(list(self.rating.scores.keys()), ["2", "4"])
        self.assertEqual(self.rating.amountOfRatings, 2)  
        self.assertEqual(self.rating.ipaddresses, ["127.0.0.1"])
                                                
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestRating))        
    suite.addTest(makeSuite(TestAnonymousRating))            
    return suite
